local _, FH = ...
local L = FH.L

local msq, msqGroups = nil, {}
if LibStub then
	msq = LibStub("Masque",true)
	if msq then
		msqGroups = {
			FarmhandTools = msq:Group("Farmhand","Tools"),
			FarmhandSeeds = msq:Group("Farmhand","Seeds"),
			FarmhandPortals = msq:Group("Farmhand","Portals"),
		}
	end
end

function Farmhand_Initialize()
	
	if FarmhandData == nil then
		FarmhandData = {
			X = 0,
			Y = -UIParent:GetHeight() / 5,
			Scale = 1,
			ToolsLocked = false,
			PrintScannerMessages = true,
			PlayScannerSounds = true,
			AlwaysShowPortals = true,
		}
	end

	FarmhandToolsLockOption:SetChecked(FarmhandData.ToolsLocked)
	FarmhandMessagesOption:SetChecked(FarmhandData.PrintScannerMessages)
	FarmhandSoundsOption:SetChecked(FarmhandData.PlayScannerSounds)
	FarmhandPortalsOption:SetChecked(FarmhandData.AlwaysShowPortals)
	Farmhand_SetPortalsOption(FarmhandData.AlwaysShowPortals)

	--print("Intial X="..FarmhandData.X.." Y="..FarmhandData.Y)
	Farmhand:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
	
	FarmhandSeeds.Update = Farmhand_UpdateBar
	FarmhandTools.Update = Farmhand_UpdateBar
	FarmhandPortals.Update = Farmhand_UpdateBar

end

function Farmhand_MerchantEvent(MerchantOpen)
	FH.MerchantOpen = MerchantOpen
end

function Farmhand_ZoneChanged()

	local Zone, SubZone = GetZoneText(), GetSubZoneText()

	local InSunsong = SubZone == L["Sunsong Ranch"]
	local InMarket = SubZone == L["The Halfhill Market"]
	local InHalfhill = InSunsong or InMarket or SubZone == L["Halfhill"] or Zone == L["Halfhill"]
	
	if not InHalfhill and not FH.InHalfhill then return end

	local LeavingHalfhill = not InHalfhill and FH.InHalfhill

	local EnteringSunsong = InSunsong and not FH.InSunsong
	local LeavingSunsong = not InSunsong and FH.InSunsong

	local EnteringMarket = InMarket and not FH.InMarket
	local LeavingMarket = not InMarket and FH.InMarket
	
	
	if (LeavingSunsong or LeavingMarket) and not (EnteringSunsong or EnteringMarket) then
		--print("Leaving Sunsong area. Hiding Farmhand")
		Farmhand:UnregisterEvent("BAG_UPDATE")
		Farmhand:UnregisterEvent("MERCHANT_SHOW")
		Farmhand:UnregisterEvent("MERCHANT_CLOSED")
		FarmhandSeeds:Hide()
		FarmhandTools:Hide()
		if FarmhandData.AlwaysShowPortals then FarmhandPortals:Hide() end
		Farmhand:Hide()
	end
	
	if (EnteringSunsong or EnteringMarket) and not (FH.InSunsong or FH.InMarket) then
		--print("Entering Sunsong area. Updating Farmhand.")
		Farmhand:RegisterEvent("BAG_UPDATE")
		Farmhand:RegisterEvent("MERCHANT_SHOW")
		Farmhand:RegisterEvent("MERCHANT_CLOSED")
		Farmhand:Show()
		FarmhandSeeds:Show()
	end
	
	if EnteringSunsong then
		FarmhandTools:Show()
		if FarmhandData.AlwaysShowPortals then FarmhandPortals:Show() end
	elseif LeavingSunsong then
		FarmhandTools:Hide()
		if FarmhandData.AlwaysShowPortals then FarmhandPortals:Hide() end
	end
	
	if EnteringSunsong or EnteringMarket then
		Farmhand:Show()
		Farmhand_Update()
	end
	
	if LeavingHalfhill then
		Farmhand_DropTools()
	end
	
	FH.InHalfhill = InHalfhill
	FH.InMarket = InMarket
	FH.InSunsong = InSunsong
	
end

function Farmhand_ItemPreClick(Button,MouseButton,Down)
	if Down and not InCombatLockdown() then
		local Bag, Slot = Farmhand_FindItemInBags(Button.ItemID)
		if IsShiftKeyDown() then
			Button:SetAttribute("type",nil)
		elseif FH.InSunsong and UnitName("target") ~= L["Tilled Soil"] and Button:GetParent():GetName() == "FarmhandSeeds" then
			Button:SetAttribute("type","macro")
			Button:SetAttribute("macrotext","/targetexact "..L["Tilled Soil"].."\n/use "..Bag.." "..Slot)
		else
			Button:SetAttribute("type","item")
			Button:SetAttribute("item",Bag.." "..Slot)
		end
	end
end

function Farmhand_ItemPostClick(Button,MouseButton,Down)
	if Down then return end
	if not InCombatLockdown() then
		Button:SetAttribute("type","item")
		Button:SetAttribute("item","item:"..Button.ItemID)
		Button:SetAttribute("shift-item*","")
	end
end

function Farmhand_ItemOnEnter(Button)
	if FH.MerchantOpen and Button:GetParent():GetName() == "FarmhandSeeds" then
		ShowContainerSellCursor(Button.Bag,Button.Slot)
	end
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetBagItem(Button.Bag,Button.Slot)
	GameTooltip:Show()
end

function Farmhand_ItemOnLeave(Button)
	if FH.MerchantOpen and Button:GetParent():GetName() == "FarmhandSeeds" then
		ResetCursor()
	end
	GameTooltip_Hide()
end

function Farmhand_ButtonOnMouseDown(Button, MouseButton)
	if IsShiftKeyDown() and MouseButton == "LeftButton" and not Farmhand.isMoving then
		_,_,_, Farmhand.InitialOffsetX, Farmhand.InitialOffsetY = Farmhand:GetPoint(1)
--		print("InitialOffsetX: "..Farmhand.InitialOffsetX.." InitialOffsetY: "..Farmhand.InitialOffsetY)
		Farmhand:StartMoving()
		_,_,_, Farmhand.PickupOffsetX, Farmhand.PickupOffsetY = Farmhand:GetPoint(1)
--		print("PickupOffsetX: "..Farmhand.PickupOffsetX.." PickupOffsetY: "..Farmhand.PickupOffsetY)
		Farmhand.isMoving = true
	elseif MouseButton == "RightButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		FarmhandData.X, FarmhandData.Y = 0, - UIParent:GetHeight() / 5
		Farmhand:ClearAllPoints()
		Farmhand:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
	end
end

function Farmhand_ButtonOnMouseUp(Button, MouseButton)
	if MouseButton == "LeftButton" and Farmhand.isMoving then
		local _,_,_, DropOffsetX, DropOffsetY = Farmhand:GetPoint(1)
--		print("DropOffsetX: "..DropOffsetX.." DropOffsetY: "..DropOffsetY)
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		FarmhandData.X = DropOffsetX - Farmhand.PickupOffsetX + Farmhand.InitialOffsetX
		FarmhandData.Y = DropOffsetY - Farmhand.PickupOffsetY + Farmhand.InitialOffsetY
--		print("FinalOffsetX: "..FarmhandData.X.." FinalOffsetY: "..FarmhandData.Y)
		Farmhand:ClearAllPoints()
		Farmhand:SetPoint("Center",UIParent,FarmhandData.X,FarmhandData.Y)
   end
end

function Farmhand_ButtonOnHide()
	if Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		Farmhand:ClearAllPoints()
		Farmhand:SetPoint("Center", UIParent, "Center", FarmhandData.X, FarmhandData.Y)
	end
end

function Farmhand_SetLockToolsOption(Value)
	FarmhandData.ToolsLocked = Value
end

function Farmhand_SetMessagesOption(Value)
	FarmhandData.PrintScannerMessages = Value
end

function Farmhand_SetSoundsOption(Value)
	FarmhandData.PlayScannerSounds = Value
end

function Farmhand_SetPortalsOption(Value)
	FarmhandData.AlwaysShowPortals = Value
end

function Farmhand_UpdateBar(Bar)
	local Last
	local Shown = 0
	local ButtonSpacing = 6
	
	--print("UpdateBar "..Bar:GetName().." with "..#Bar.Buttons.." items.")

	for _, Button in ipairs(Bar.Buttons) do
		local ItemCount = GetItemCount(Button.ItemID)
		if ItemCount > 0 or Button.AlwaysShow then
			if Shown % 8 == 0 then
				local Row = math.floor(Shown/8)
				Button:SetPoint("TopLeft",Bar,"TopLeft", ButtonSpacing/2, -ButtonSpacing/2 - Row*Button:GetHeight() - Row*ButtonSpacing)
				Last = Button
			else
				Button:SetPoint("TopLeft",Last,"TopRight",ButtonSpacing,0)
				Last = Button
			end
			if Button.ItemID then
				Button.Bag, Button.Slot = Farmhand_FindItemInBags(Button.ItemID)
				if Button.Bag and Button.Slot then
					local Icon = GetContainerItemInfo(Button.Bag,Button.Slot)
					Button.Icon:SetTexture(Icon)
				end
				if Bar:GetName() ~= "FarmhandTools" then
					if ItemCount > 999 then
						Button.Count:SetText("***")
					else
						Button.Count:SetText(ItemCount)
					end
				end
			end
			Button:Show()
			Shown = Shown + 1
		else
			Button:Hide()
		end
	end

	if msqGroups[Bar:GetName()] then msqGroups[Bar:GetName()]:ReSkin() end

	local Width, Height
	if Last then
		Width = math.min(8,Shown) * (Last:GetWidth() + ButtonSpacing) 
		Height = math.ceil(Shown/8) * (Last:GetHeight() + ButtonSpacing)
	end
	Bar:SetWidth(Width or (32 + ButtonSpacing))
	Bar:SetHeight(Height or (32 + ButtonSpacing))
end

function Farmhand_Update()
	
	FarmhandSeeds:Update()
	FarmhandTools:Update()
	FarmhandPortals:Update()
	
	local SBH = FarmhandSeeds:GetHeight() * FarmhandSeeds:GetScale()
	local TBH = FarmhandTools:GetHeight() * FarmhandTools:GetScale()
	local PBH = FarmhandPortals:GetHeight() * FarmhandPortals:GetScale()
	local FHH = SBH + TBH + PBH -- + Farmhand.Backdrop:GetBackdrop().insets.top + Farmhand.Backdrop:GetBackdrop().insets.bottom
	Farmhand:SetHeight(FHH)
	
	local SBW = FarmhandSeeds:GetWidth() * FarmhandSeeds:GetScale()
	local TBW = FarmhandTools:GetWidth() * FarmhandTools:GetScale()
	local PBW = FarmhandPortals:GetWidth() * FarmhandPortals:GetScale()
	local FHW = max(SBW,TBW,PBW) -- + Farmhand.Backdrop:GetBackdrop().insets.left + Farmhand.Backdrop:GetBackdrop().insets.right
	Farmhand:SetWidth(FHW)

end

function Farmhand_CropScannerPreClick(Button,MouseButton,Down)
	if Down and not InCombatLockdown() then
		if IsShiftKeyDown() then
			Button:SetAttribute("type",nil)
		else
			Button:SetAttribute("type","click")
		end
	end
end

function Farmhand_CropScannerCheckForTarget()
	if UnitExists("target") then
		CropName = UnitName("target")
		Icon = ICON_LIST[GetRaidTargetIndex("target")] and ICON_LIST[GetRaidTargetIndex("target")].."0|t" or ""
		local msg = L["Crop Scanner found:"].." "..Icon.." "..CropName
		tinsert(FarmhandScanButton.ScannerOutput,1,msg)
	end
end

function Farmhand_CropScannerPostClick(Button,MouseButton,Down)
	if Down then return end
	if not Button:GetAttribute("type") then 
		if not InCombatLockdown() then
			Button:SetAttribute("type","click")
		end
		return 
	end
	if #Button.ScannerOutput == 0 then
		if FarmhandData.PrintScannerMessages then
			print(L["Crop Scanner finished."].." "..L["The crops are looking good!"])
			RaidNotice_AddMessage(RaidBossEmoteFrame,L["The crops are looking good!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
		end
		if FarmhandData.PlayScannerSounds then
			PlaySound("QUESTCOMPLETED","SFX")
		end
	else
		if FarmhandData.PrintScannerMessages then
			for _, msg in ipairs(Button.ScannerOutput) do
				print(msg)
			end
			RaidNotice_AddMessage(RaidBossEmoteFrame,L["Some crops need attention!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
		end
		if FarmhandData.PlayScannerSounds then
			PlaySound("igQuestFailed","SFX")
		end
		wipe(Button.ScannerOutput)
	end
end

function Farmhand_ScanButtonOnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:ClearLines()
	GameTooltip:AddLine(L["Crop Scanner"],0,255,0,false)
	GameTooltip:AddLine(L["Scan your farm for crops or soil that need attention."],255,255,255,true)
	GameTooltip:Show()
end

function Farmhand_ScanButtonOnLeave()
	GameTooltip_Hide()
end

function Farmhand_DropTools()
	if FarmhandData.ToolsLocked == true then
		print(L["Leaving Halfhill."].." "..L["Tools are Locked."])
	else
		ClearCursor()
		for _, ItemID in ipairs(FH.Tools) do
			local Bag, Slot = Farmhand_FindItemInBags(ItemID)
			if Bag and Slot then
				PickupContainerItem(Bag,Slot)
				if CursorHasItem() then
					local _, ID, Link = GetCursorInfo()
					if ID == ItemID then
						print(L["Leaving Halfhill."].." "..L["Dropping"].." "..Link..".")
						DeleteCursorItem()
					end
				end
			end
		end
	end
end

function Farmhand_FindItemInBags(item)
	for container = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(container)
		if numSlots then
			for slot = 1, numSlots do
				if item == GetContainerItemID(container, slot) then
					return container, slot
				end
			end
		end
	end
end

FH.PostCombatQueue = {}
function Farmhand_RunAfterCombat(Func,Args)
	if InCombatLockdown() then
		Farmhand:RegisterEvent("PLAYER_REGEN_ENABLED")
		tinsert(FH.PostCombatQueue,{Func,Args})
		return
	else
		Func(Args and unpack(Args))
	end
end
function Farmhand_CombatEnded()
	for _, Item in ipairs(FH.PostCombatQueue) do
		Item.Func(Item.Args and unpack(Item.Args))
	end
	wipe(FH.PostCombatQueue)
end
